#
#     Main program for Example 4.1: Randomized block design
#
#     ===========================================================
#
#     The basic ANOVA in Table 1 is obtained and an illustration of
#     an AIC_c v Tr plot is given.  Other plots can be obtained by
#     altering the parameters ndx, bdeg and pord and running setup.s;
#     a little experimenting will soon reveal the appropriate range
#     of lambda for these parameters
#
data <- scan("4.1.dat")
Pick <- seq(1, 215 , by=2) 
Yield <- data[Pick]
Variety <- factor(data[Pick + 1])
Block <- factor(rep(1:3,rep(36,3)))
Wheel <- factor(rep(c(1,2,3,3,2,1), 18))
#
#     Obtain treatment, block and wheel matrix
#     and remove block effects from X
#     Fit basic fixed effects model Block + Variety + Wheel + linear
#
options(contrasts = c("contr.sum"))
X <- model.matrix(Yield ~ Block + Variety + Wheel)
X <- X[ ,4:40]
Lin.1 <- c(1:36, rep(0,36), rep(0,36))
Lin.2 <- c(rep(0,36), 1:36, rep(0,36))
Lin.3 <- c(rep(0,36), rep(0,36), 1:36)
aov.lin <- aov(Yield ~ Block + Variety + Wheel + Lin.1 + Lin.2 + Lin.3)
#
#     Set up basic parameters
#
x <- 1:36
xl <- 0.99999
xr <- 36.00001
ndx <- 10
bdeg <- 3
pord <- 2
#
#    Load functions and run setup
#
source("functions.s")
source("4.1.setup.s")
#
#    Make Aic.c v tr plot
#
par(mfrow = c(1,1))
Lambda <- c(seq(5,20,by=1), 21:40,seq(50, 100, by = 10))
Aic <- rep(0, length(Lambda))
Trace <- rep(0, length(Lambda))
for (i in 1:length(Lambda)){
  Ans <- Calc(XpB, BigDtD, Yield, Lambda[i])
  Aic[i] <- Ans$Aic.c
  Trace[i] <- Ans$Trace
}
plot(Trace, Aic, type = "l")
#
#    =================================================
#
#    Now do REML fit for selected P-spline parameters
#
#    Set up basic parameters
#
ndx <- 5
bdeg <- 3
pord <- 1
#
#   Run setup again
#
source("4.1.setup.s")
#
Lambda.init <- 1
Opt.Reml <- ms(~ - P.Reml.QR(bigX, bigBZ, Yield, Lambda),
                 start = list(Lambda = Lambda.init))
L.Reml <- Opt.Reml$par
S2.Reml <- CondS2(bigX, bigBZ, Yield, L.Reml)
Ans.Reml <- Calc(XpB, BigDtD, Yield, L.Reml)
loglik.2 <- P.Reml.QR(bigX, bigBZ, Yield, L.Reml)
Trace.Reml <- Ans.Reml$Trace
Coef.Reml <- Ans.Reml$Coef
Fit.smooth <- BigB %*% Coef.Reml[38:61]
Fit.var.wheel <- X %*% Coef.Reml[1:37]
#
#    Answer: \hat \sigma^2 = 8.373763, 
#            \hat \lambda = 0.8198504
#            -2*loglik = 507.3627
#            Trace(H) = 47.24
#
#    Conduct LRT
#
loglik.1 <- P.Reml.QR(bigX, bigBZ, Yield, 10^9)
Test1 <- -2*(loglik.1 - loglik.2)   #    LRT = 11.78
#
#    ============================================
#
#                 Separate lambda
#
Opt.Reml.3 <- ms(~ - P.Reml.3(bigX, BZ, Yield, c(Lambda1, 10^7, Lambda3)),
                 start = list(Lambda1 = 0.2, Lambda3 = 0.6))
L.Reml.3 <- c(Opt.Reml.3$par[1], 10^7, Opt.Reml.3$par[2])
bigBZ.3 <- dbind(list( BZ/sqrt(L.Reml.3[1]), BZ/sqrt(L.Reml.3[2]),
                    BZ/sqrt(L.Reml.3[3]) ))
S2.Reml.3 <- CondS2(bigX, bigBZ.3, Yield, 1)
BigDtD.3 <- Augment(X, dbind(list( L.Reml.3[1]*DtD, L.Reml.3[2]*DtD,
                                  L.Reml.3[3]*DtD )))
Ans.Reml.3 <- Calc(XpB, BigDtD.3, Yield, 1)
loglik.3 <- P.Reml.QR(bigX, bigBZ.3, Yield, 1)
Test2 <- -2*(loglik.2 - loglik.3)       #    LRT  =  3.50
Trace.Reml.3 <- Ans.Reml.3$Trace
Coef.Reml.3 <- Ans.Reml.3$Coef
Fit.smooth.3 <- BigB %*% Coef.Reml.3[38:61]
Fit.var.wheel.3 <- X %*% Coef.Reml.3[1:37]
#
#    Answer: \hat \sigma^2 = 7.921613, 
#            \hat \lambda.1 = 0.1825409
#            \hat \lambda.2 = Inf (= 10^7)
#            \hat \lambda.3 = 0.6422672, 
#            -2*loglik = 503.8656
#            Trace(H) = 46.2363
#
SD <- sqrt( S2.Reml )
plot3(Yield, Fit.var.wheel, Fit.smooth, Fit.var.wheel.3, Fit.smooth.3, 36)
plot3.res(Yield, Fit.var.wheel, Fit.smooth, SD, 36)






