#
#     Main program for Example 4.2: Heteroscedastic errors
#
# ====================================================================
#
source("functions.s")
Data <- scan(file="4.2.dat")
Pick <- 1:133
Time <- Data[2*Pick - 1]
Head <- Data[2*Pick]
Order.Time <- order(Time)
Time <- Time[Order.Time]
Head <- Head[Order.Time]
x <- Time
y <- Head
#
#    Set up Pspline parameters
#
xl <- 2
xr <- 60
ndx <- 20
bdeg <- 3
pord <- 2
#
#     Compute B, G, BG, Z and BZ
#
B <- bspline(x, xl, xr, ndx, bdeg)
G <- matrix(1, nrow = ncol(B))
for (i in 1:(pord - 1)) G <- cbind(G, (1:ncol(B))^i)
BG <- B %*% G
D <- diag( ncol(B) )
for (k in 1:pord) D <- diff(D)
DtD <- t(D) %*% D
Z <- t(D) %*% solve(D %*% t(D))
BZ <- B %*% Z
#
#   ==================================================
#
#   Unweighted fit
#
Opt.Reml <- ms( ~ -P.Reml.QR(BG, BZ, y, Lambda), 
      start = list(Lambda = 1), trace = T )
L.Reml <- Opt.Reml$par
S2.Reml <- CondS2(BG, BZ, y, L.Reml)
#
#    Lambda: 0.3277862; S.2: 511.7154
#
Fitted <- Calc(B, DtD, y, L.Reml)$Fitted
par(mfrow = c(1,1))
plot(Time, Head, xlab = "Time (ms)", ylab = "Acceleration (g)" )
lines(Time, Fitted)
#
#   =============================================================
#
#   Weighted fit.   First, get weights
#
sd <- sqrt(S2.Reml)
Resid <- as.vector( (Head - Fitted)/sd)
plot(x, Resid^2) 
Lres <- log(Resid^2)
plot(Time, Lres)
#
#    Smooth Lres - first with Psplines
#
Opt.Reml.Var <- ms( ~ -P.Reml.QR(BG, BZ, Lres, Lambda), 
      start = list(Lambda = 1), trace = T )
L.Reml.Var <- Opt.Reml.Var$par
S2.Reml.Var <- CondS2(BG, BZ, Lres, L.Reml.Var)
#
#      Lambda: 19.56191;     S.2: 3.614136
#
Fitted.Var <- Calc(B, DtD, Lres, L.Reml.Var)$Fitted
plot(Time, Lres)
lines(Time, Fitted.Var)
#
wt <- exp(Fitted.Var)
wt <- wt/mean(wt)
#
#    Second by Moving Average (Silverman)
#
n <- length(x)
Resid.2 <- Resid^2
k <- 5
wt.silver <- rep(0,n)
for (i in 1:n){
  m.i <- max(1, i - k)
  n.i <- min(n, i + k)
  wt.silver[i] <- sum( Resid.2[m.i:n.i]/(n.i - m.i + 1) )
}
wt.silver <- wt.silver/mean(wt.silver)
#
par(mfrow = c(1,2))
plot(Time, Resid^2, xlab = "Time (ms)", ylab = "Residual^2")
plot(Time, wt.silver, ylab = "Weights^(-1)", xlab = "Time (ms)")
lines(Time, wt)
#
#    =================================================
#
#    Weighted fit: Pinheiro & Bates p202-3
#
#    Pspline weights
#
Wt.half <- c(wt^(-0.5))
y.star <- Wt.half * y
X.star <- Wt.half * BG
Z.star <- Wt.half * BZ
Opt.Reml.Wt <- ms( ~ -P.Reml.QR(X.star, Z.star, y.star, Lambda), 
      start = list(Lambda = 1), trace = T )
L.Reml.Wt <- Opt.Reml.Wt$par
S2.Reml.Wt <- CondS2(X.star, Z.star, y.star, L.Reml.Wt)
#
#    Lambda: 0.3436818;    S.2: 508.101 
#        
#    ================================================
#
#    Silverman weights    
#
Wt.silver.half <- c(wt.silver^(-0.5))
y.silver.star <- Wt.silver.half * y
X.silver.star <- Wt.silver.half * BG
Z.silver.star <- Wt.silver.half * BZ
Opt.Reml.Silver <- ms( ~ -P.Reml.QR(X.silver.star, Z.silver.star,
                                    y.silver.star, Lambda),  
      start = list(Lambda = 1), trace = T )
L.Reml.Silver <- Opt.Reml.Silver$par
S2.Reml.Silver <- CondS2(X.silver.star, Z.silver.star, y.silver.star,
                         L.Reml.Silver)
#
#    Lambda: 0.2817548;     S.2: 445.6294 
#
#    Use (2.5) in paper
#
Vm1 <- diag(1/c(wt))
H.V <- B %*% solve(t(B) %*% Vm1 %*% B + L.Reml.Wt * DtD, t(B) %*% Vm1)
Fitted.Wt <- c(H.V %*% y)
#
#  or by   1/Wt.half*c(Calc(Wt.half*B, DtD, y.star, L.Reml.Wt)$Fitted)
#
SE.fit <- sqrt(S2.Reml.Wt * diag(H.V %*% diag(c(wt)) %*% t(H.V)) )
par(mfrow = c(1,1))
plot(Time, Head, ylab = "Acceleration (g)", xlab = "Time (ms)")
lines(Time, Fitted.Wt, lwd = 2)
lines(Time, Fitted.Wt + 1.96*SE.fit, lty = 2, lwd = 2)
lines(Time, Fitted.Wt - 1.96*SE.fit, lty = 2, lwd = 2)


