#
#     Main program for example 4.3: model with trend
#         and serially correlated errors
#
source("4.3.functions.s")
wood <- scan(file="4.3.dat")
Sampling <- 1:320
#
#    Set up Pspline parameters
#
xl <- 0.99999
xr <- 320.00001
ndx <- 40
bdeg <- 3
pord <- 2
#
wood.setup <- Pspline.setup(Sampling,xl, xr, ndx, bdeg, pord)
B. <- wood.setup$B.
Z. <- wood.setup$Z.
Z. <- B.%*%Z.
D. <- wood.setup$D.
X. <- wood.setup$tX
#
#    =======================================
# 
#    Model with trend and independent errors
#
#    Initial values and estimation
#
Init.v <- c(0.15, 13)
Reml.fit1 <- ms(~  -P.Reml.QR.cor(X., Z., wood, lambda, 0, 0, 1),
    start=c(lambda=Init.v[1]))
#
#    Results: lambda=0.162
#
lambda.1 <- Reml.fit1$param[1]
n <- length(wood)
#
#     Fitted values
#
curve <- Fit.Val.cor(X., Z., wood, lambda.1, 0, 0, 1)
plot(Sampling, wood)
lines(Sampling,curve$fit, lty=2)
#
# =======================================
# 
#    Model with trend and AR1 error structure
#
#    Initial values and estimation
#
Init.v2 <- c(300, 0.9)
Reml.fit2 <- ms(~-P.Reml.QR.cor(X.,Z.,wood, lambda, rho1,0,1),
                start = c(lambda=Init.v2[1], rho1=Init.v2[2]))
#
#    Results: lambda=256.65, sigma^2=28.4, rho1=0.804
#
# =======================================
#
#    Model with trend and AR2 error structure
#
#    Initial values and estimation
#
Init.v3 <- c(150, 0.98, -0.2)
Reml.fit3<-ms(~-P.Reml.QR.cor(X.,Z.,wood, lambda, rho1, rho2, 2),
     start = c(lambda=Init.v3[1], rho1=Init.v3[2], rho2=Init.v3[3]))
#
#    Results: lambda=159.78, sigma^2=26.21, rho1=0.975, rho2=-0.238
#    
lambda.3 <- Reml.fit3$param[1]
rho1.3 <- Reml.fit3$param[2]
rho2.3 <- Reml.fit3$param[3]
#
#     Fitted values, confidence intervals and plot
#
curve.3 <- Fit.Val.cor(X., Z.,wood, lambda.3, rho1.3, rho2.3, 2)
ci <- ci.cor(B., wood, lambda.3, rho1.3, rho2.3, D.,curve.3$sigma.sq,2)
lines(Sampling, curve.3$fit)
lines(Sampling,curve.3$fit+ci, lty=3)
lines(Sampling,curve.3$fit-ci, lty=3)
